#include "BehaviourPlayer.h"
#include "Level.h"
#include "LD.h"



BehaviourPlayer::BehaviourPlayer() :
  m_pWeapon( NULL ),
  m_WeaponSwing( 0 ),
  m_LookAngle( 0.0f ),
  m_Frozen( false ),
  m_CurWeapon( LD::OT_INVALID ),
  m_DX( 0 ),
  m_DY( 1 ),
  m_AnimPos( 0 ),
  m_HP( 3 ),
  m_HPMax( 3 )
{
}



void BehaviourPlayer::UpdateFixed( Level& Level, GameObject* pObj )
{

  if ( Level.m_Done )
  {
    return;
  }

  if ( m_Frozen )
  {
    return;
  }

  if ( m_WeaponSwing == 0 )
  {
    int     speed = 3;
    int     dx = 0;
    int     dy = 0;
    if ( theApp.m_bKeyPressed[VK_LEFT] )
    {
      Level.WalkObject( pObj, -speed, 0 );
      dx = -1;
    }
    if ( theApp.m_bKeyPressed[VK_RIGHT] )
    {
      Level.WalkObject( pObj, speed, 0 );
      dx = 1;
    }
    if ( theApp.m_bKeyPressed[VK_UP] )
    {
      Level.WalkObject( pObj, 0, -speed );
      dy = -1;
    }
    if ( theApp.m_bKeyPressed[VK_DOWN] )
    {
      Level.WalkObject( pObj, 0, speed );
      dy = 1;
    }
    // who the F ever uses radians?
    if ( ( dx != 0 )
    ||   ( dy != 0 ) )
    {
      m_DX = dx;
      m_DY = dy;
      m_LookAngle = atan2f( (float)dy, (float)dx ) * 180.0f / 3.1415926f;

      m_AnimPos = ( ( m_AnimPos + 1 ) % 10 );
      if ( m_DX == 1 )
      {
        pObj->m_TexSection = (LD::TextureTypes)( LD::TEX_PLAYER_R_1 + m_AnimPos / 5 );
      }
      else if ( m_DX == -1 )
      {
        pObj->m_TexSection = (LD::TextureTypes)( LD::TEX_PLAYER_L_1 + m_AnimPos / 5 );
      }
      else if ( m_DY == 1 )
      {
        pObj->m_TexSection = (LD::TextureTypes)( LD::TEX_PLAYER_D_1 + m_AnimPos / 5 );
      }
      else if ( m_DY == -1 )
      {
        pObj->m_TexSection = (LD::TextureTypes)( LD::TEX_PLAYER_U_1 + m_AnimPos / 5 );
      }
    }
    else
    {
      if ( m_DX == 1 )
      {
        pObj->m_TexSection = LD::TEX_PLAYER_R;
      }
      else if ( m_DX == -1 )
      {
        pObj->m_TexSection = LD::TEX_PLAYER_L;
      }
      else if ( m_DY == 1 )
      {
        pObj->m_TexSection = LD::TEX_PLAYER_D;
      }
      else if ( m_DY == -1 )
      {
        pObj->m_TexSection = LD::TEX_PLAYER_U;
      }
    }

    if ( !m_Items.empty() )
    {
      if ( ( ( theApp.m_bKeyPressed['Y'] )
      &&     ( theApp.m_bKeyReleased['Y'] ) )
      ||   ( ( theApp.m_bKeyPressed['Z'] )
      &&     ( theApp.m_bKeyReleased['Z'] ) ) )
      {
        theApp.m_bKeyReleased['Y'] = false;
        theApp.m_bKeyReleased['Z'] = false;
        std::map<LD::ObjectType,int>::iterator    itItems( m_Items.find( m_CurWeapon ) );
        if ( itItems == m_Items.end() )
        {
          // find first weapon
          itItems = m_Items.begin();
          while ( itItems != m_Items.end() )
          {
            if ( ( itItems->first == LD::OT_AXE )
            ||   ( itItems->first == LD::OT_SWORD )
            ||   ( itItems->first == LD::OT_GOLDEN_SWORD )
            ||   ( itItems->first == LD::OT_HAMMER ) )
            {
              m_CurWeapon = itItems->first;
              break;
            }
            ++itItems;
          }
        }
        else
        {
          do
          {
            ++itItems;
            if ( itItems == m_Items.end() )
            {
              itItems = m_Items.begin();
            }
            if ( ( itItems->first == LD::OT_AXE )
            ||   ( itItems->first == LD::OT_SWORD )
            ||   ( itItems->first == LD::OT_GOLDEN_SWORD )
            ||   ( itItems->first == LD::OT_HAMMER ) )
            {
              m_CurWeapon = itItems->first;
              break;
            }
          }
          while ( itItems->first != m_CurWeapon );
        }
      }
    }
  }

  if ( m_WeaponSwing == 0 )
  {
    if ( theApp.m_bKeyPressed[VK_CONTROL] )
    {
      if ( ( m_CurWeapon != LD::OT_INVALID )
      &&   ( m_Items[m_CurWeapon] > 0 ) )
      {
        m_WeaponSwing = 10;
        m_pWeapon = new GameObject( pObj->m_X, pObj->m_Y, m_CurWeapon );

        m_pWeapon->m_X += (int)( 20.0f * cosf( m_LookAngle * 3.1415926f / 180.0f ) );
        m_pWeapon->m_Y += (int)( 20.0f * sinf( m_LookAngle * 3.1415926f / 180.0f ) );
        m_pWeapon->m_Angle = -m_LookAngle;

        Level.AddObject( m_pWeapon );

        if ( ( m_CurWeapon == LD::OT_SWORD )
        ||   ( m_CurWeapon == LD::OT_GOLDEN_SWORD ) )
        {
          theApp.PlaySound( LD::SOUND_SWORD );
        }
        else if ( m_CurWeapon == LD::OT_AXE )
        {
          theApp.PlaySound( LD::SOUND_AXE );
        }
        else if ( m_CurWeapon == LD::OT_HAMMER )
        {
          theApp.PlaySound( LD::SOUND_HAMMER );
        }
      }
    }
  }
  else
  {
    m_pWeapon->m_Angle -= 10.0f;
    m_pWeapon->m_X = (int)( pObj->m_X + 20.0f * cosf( m_pWeapon->m_Angle * 3.1415926f / 180.0f ) );
    m_pWeapon->m_Y = (int)( pObj->m_Y - 20.0f * sinf( m_pWeapon->m_Angle * 3.1415926f / 180.0f ) );

    --m_WeaponSwing;
    if ( m_WeaponSwing == 0 )
    {
      m_pWeapon->m_RemoveMe = true;
      m_pWeapon = NULL;
    }
  }

  Level.m_OffsetX = pObj->m_X;
  Level.m_OffsetY = pObj->m_Y;

}



void BehaviourPlayer::Update( Level& Level, float ElapsedTime, GameObject* pObj )
{
}



void BehaviourPlayer::OnEvent( Level& Level, const std::string& Event, GameObject* pObj, GameObject* pOtherObject )
{

  if ( Event == "Collect" )
  {
    if ( pOtherObject->m_Type == LD::OT_SWORD_COLLECTABLE )
    {
      m_Items[LD::OT_SWORD]++;
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
    }
    else if ( pOtherObject->m_Type == LD::OT_AXE_COLLECTABLE )
    {
      m_Items[LD::OT_AXE]++;
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
    }
    else if ( pOtherObject->m_Type == LD::OT_HAMMER_COLLECTABLE )
    {
      m_Items[LD::OT_HAMMER]++;
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
    }
    else if ( pOtherObject->m_Type == LD::OT_GOLDEN_SWORD_COLLECTABLE )
    {
      m_Items[LD::OT_GOLDEN_SWORD]++;
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
    }
    else if ( pOtherObject->m_Type == LD::OT_COIN )
    {
      m_Items[pOtherObject->m_Type]++;

      theApp.PlaySoundA( LD::SOUND_PICK_COIN );
    }
    else if ( pOtherObject->m_Type == LD::OT_WATER_SHOES )
    {
      m_Items[pOtherObject->m_Type]++;
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
    }
    else if ( pOtherObject->m_Type == LD::OT_HEART_COLLECTABLE )
    {
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
      pObj->OnEvent( Level, "Heal", pObj, pOtherObject );
    }
    else if ( pOtherObject->m_Type == LD::OT_HEART_CONTAINER )
    {
      theApp.PlaySoundA( LD::SOUND_PICK_ITEM );
      pObj->OnEvent( Level, "Heal", pObj, pOtherObject );
    }
    if ( m_CurWeapon == LD::OT_INVALID )
    {
      std::map<LD::ObjectType,int>::iterator itItems = m_Items.begin();
      while ( itItems != m_Items.end() )
      {
        if ( ( itItems->first == LD::OT_AXE )
        ||   ( itItems->first == LD::OT_SWORD )
        ||   ( itItems->first == LD::OT_GOLDEN_SWORD )
        ||   ( itItems->first == LD::OT_HAMMER ) )
        {
          m_CurWeapon = itItems->first;
          break;
        }
        ++itItems;
      }
    }
  }
  else if ( Event == "LevelChanged" )
  {
    m_pWeapon = NULL;
    m_WeaponSwing = 0;
  }
  else if ( Event == "Killed" )
  {
    if ( m_pWeapon != NULL )
    {
      m_pWeapon->m_RemoveMe = true;
    }
    m_pWeapon = NULL;
    Level.m_pPlayer = NULL;
  }
  else if ( Event == "Freeze" )
  {
    m_Frozen = true;
  }
  else if ( Event == "Thaw" )
  {
    m_Frozen = false;
  }

}