#include "GameObject.h"
#include "Level.h"
#include "LD.h"

#include "BehaviourPlayer.h"
#include "BehaviourOcto.h"
#include "BehaviourCoin.h"
#include "BehaviourKillable.h"
#include "BehaviourChest.h"
#include "BehaviourSpillBonus.h"
#include "BehaviourPushback.h"
#include "BehaviourShortLived.h"
#include "BehaviourParticle.h"
#include "BehaviourShoot.h"
#include "BehaviourShot.h"
#include "BehaviourAnim.h"



GameObject::GameObject( int X, int Y, LD::ObjectType Type ) :
  m_X( X ),
  m_Y( Y ),
  m_XFract( 0.0f ),
  m_YFract( 0.0f ),
  m_RemoveMe( false ),
  m_Type( Type ),
  m_ID( 0 ),
  m_Rotateable( false ),
  m_Angle( 0.0f ),
  m_Color( 0xffffffff )
{

  m_DisplayOffset.x = 0;
  m_DisplayOffset.y = 0;
  SetRect( &m_Bounds, -10, -10, 10, 10 );
  switch ( m_Type )
  {
    case LD::OT_PLAYER:
      m_TexSection = LD::TEX_PLAYER_D;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -40;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      m_Behaviour.push_back( new BehaviourPlayer() );
      m_Behaviour.push_back( new BehaviourKillable( 3 ) );
      m_Behaviour.push_back( new BehaviourPushback( 10 ) );
      break;
    case LD::OT_BUSH:
      m_TexSection = LD::TEX_BUSH;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -18;
      m_Behaviour.push_back( new BehaviourKillable( 1 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_COIN, 0, 1 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 0, 1 ) );
      break;
    case LD::OT_TREE:
      m_TexSection = LD::TEX_TREE;
      m_DisplayOffset.x = -24;
      m_DisplayOffset.y = -46;
      SetRect( &m_Bounds, -20, -10, 20, 10 );
      break;
    case LD::OT_ENTRANCE:
      m_TexSection = LD::TEX_ENTRANCE;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -40;
      SetRect( &m_Bounds, -10, -30, 10, 10 );
      break;
    case LD::OT_OCTO:
      m_TexSection = LD::TEX_OCTO_1;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      m_Behaviour.push_back( new BehaviourOcto() );
      m_Behaviour.push_back( new BehaviourKillable( 2 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_COIN, 1, 2 ) );
      m_Behaviour.push_back( new BehaviourPushback( 20 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 0, 1 ) );
      break;
    case LD::OT_SWORD_COLLECTABLE:
      m_TexSection = LD::TEX_SWORD;
      m_DisplayOffset.x = -14;
      m_DisplayOffset.y = -14;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      break;
    case LD::OT_SWORD:
      m_TexSection = LD::TEX_SWORD;
      m_Rotateable = true;
      SetRect( &m_Bounds, -18, -18, 18, 18 );
      break;
    case LD::OT_AXE_COLLECTABLE:
      m_TexSection = LD::TEX_AXE;
      m_DisplayOffset.x = -14;
      m_DisplayOffset.y = -14;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      break;
    case LD::OT_AXE:
      m_TexSection = LD::TEX_AXE;
      m_Rotateable = true;
      SetRect( &m_Bounds, -18, -18, 18, 18 );
      break;
    case LD::OT_COIN:
      m_TexSection = LD::TEX_COIN_1;
      m_DisplayOffset.x = -12;
      m_DisplayOffset.y = -12;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      m_Behaviour.push_back( new BehaviourCoin() );
      m_Behaviour.push_back( new BehaviourAnim( LD::TEX_COIN_1, 4 ) );
      m_Behaviour.push_back( new BehaviourShortLived( 5.0f, 5.0f ) );
      break;
    case LD::OT_CHEST:
      m_TexSection = LD::TEX_CHEST;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -16, -16, 15, 15 );
      m_Behaviour.push_back( new BehaviourChest() );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_COIN, 10, 20 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 1, 3 ) );
      break;
    case LD::OT_OCTA:
      m_TexSection = LD::TEX_OCTO_1;
      m_Color = 0xffff8080;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      m_Behaviour.push_back( new BehaviourOcto() );
      m_Behaviour.push_back( new BehaviourKillable( 4 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_COIN, 1, 2 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 1, 1 ) );
      m_Behaviour.push_back( new BehaviourPushback( 15 ) );
      m_Behaviour.push_back( new BehaviourShoot( LD::OT_ENEMY_SHOT, 45 ) );
      break;
    case LD::OT_GREEN_BUSH:
      m_TexSection = LD::TEX_GREEN_BUSH;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -13, -13, 13, 13 );
      m_Behaviour.push_back( new BehaviourKillable( 1 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_LEAVES, 10, 20 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_BUSH, 1, 1 ) );
      break;
    case LD::OT_LEAVES:
      m_TexSection = LD::TEX_LEAVES;
      m_DisplayOffset.x = -3;
      m_DisplayOffset.y = -3;
      m_Behaviour.push_back( new BehaviourShortLived( 0.5f, 0.75f ) );
      m_Behaviour.push_back( new BehaviourParticle() );
      break;
    case LD::OT_ENEMY_SHOT:
      m_TexSection = LD::TEX_ENEMY_SHOT;
      m_DisplayOffset.x = -3;
      m_DisplayOffset.y = -3;
      m_Behaviour.push_back( new BehaviourShortLived( 2.5f, 2.5f ) );
      m_Behaviour.push_back( new BehaviourShot( 3 ) );
      break;
    case LD::OT_HEART_COLLECTABLE:
      m_TexSection = LD::TEX_HEART_FULL;
      m_DisplayOffset.x = -12;
      m_DisplayOffset.y = -12;
      SetRect( &m_Bounds, -12, -12, 11, 11 );
      m_Behaviour.push_back( new BehaviourCoin() );
      m_Behaviour.push_back( new BehaviourShortLived( 5.0f, 5.0f ) );
      break;
    case LD::OT_TURRET:
      m_TexSection = LD::TEX_TURRET;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -12;
      SetRect( &m_Bounds, -16, -12, 15, 11 );
      m_Behaviour.push_back( new BehaviourShoot( LD::OT_ENEMY_SHOT, 50, 1 ) );
      m_Behaviour.push_back( new BehaviourKillable( 5 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 1, 1 ) );
      break;
    case LD::OT_LINK:
      m_TexSection = LD::TEX_LINK_1;
      m_DisplayOffset.x = -12;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -12, -16, 11, 15 );
      break;
    case LD::OT_FIRE:
      m_TexSection = LD::TEX_FIRE_1;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -16, -16, 15, 15 );
      m_Behaviour.push_back( new BehaviourAnim( LD::TEX_FIRE_1, 2 ) );
      break;
    case LD::OT_HAMMER_COLLECTABLE:
      m_TexSection = LD::TEX_HAMMER;
      m_DisplayOffset.x = -14;
      m_DisplayOffset.y = -14;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      break;
    case LD::OT_HAMMER:
      m_TexSection = LD::TEX_HAMMER;
      m_Rotateable = true;
      SetRect( &m_Bounds, -18, -18, 18, 18 );
      break;
    case LD::OT_POST:
      m_TexSection = LD::TEX_POST;
      m_DisplayOffset.x = -12;
      m_DisplayOffset.y = -16;
      m_Behaviour.push_back( new BehaviourKillable( 1 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_POST_SUNK, 1, 1 ) );
      break;
    case LD::OT_POST_SUNK:
      m_TexSection = LD::TEX_POST_SUNK;
      m_DisplayOffset.x = -12;
      m_DisplayOffset.y = -16;
      break;
    case LD::OT_WATER_SHOES:
      m_TexSection = LD::TEX_WATER_SHOES;
      m_DisplayOffset.x = -11;
      m_DisplayOffset.y = -11;
      SetRect( &m_Bounds, -11, -11, 12, 12 );
      break;
    case LD::OT_GOLDEN_SWORD_COLLECTABLE:
      m_TexSection = LD::TEX_GOLDEN_SWORD;
      m_DisplayOffset.x = -14;
      m_DisplayOffset.y = -14;
      SetRect( &m_Bounds, -10, -10, 10, 10 );
      break;
    case LD::OT_GOLDEN_SWORD:
      m_TexSection = LD::TEX_GOLDEN_SWORD;
      m_Rotateable = true;
      SetRect( &m_Bounds, -18, -18, 18, 18 );
      break;
    case LD::OT_CUBE:
      m_TexSection = LD::TEX_CUBE_1;
      m_DisplayOffset.x = -24;
      m_DisplayOffset.y = -24;
      SetRect( &m_Bounds, -15, -15, 15, 15 );
      m_Behaviour.push_back( new BehaviourOcto() );
      m_Behaviour.push_back( new BehaviourKillable( 10 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_COIN, 5, 10 ) );
      m_Behaviour.push_back( new BehaviourSpillBonus( LD::OT_HEART_COLLECTABLE, 2, 2 ) );
      m_Behaviour.push_back( new BehaviourPushback( 15 ) );
      m_Behaviour.push_back( new BehaviourAnim( LD::TEX_CUBE_1, 2 ) );
      break;
    case LD::OT_HEART_CONTAINER:
      m_TexSection = LD::TEX_HEART_CONTAINER;
      m_DisplayOffset.x = -16;
      m_DisplayOffset.y = -16;
      SetRect( &m_Bounds, -15, -15, 16, 16 );
      break;
  }

}



GameObject::~GameObject()
{
  std::list<Behaviour*>::iterator   itB( m_Behaviour.begin() );
  while ( itB != m_Behaviour.end() )
  {
    Behaviour*  pBeh( *itB );
    delete pBeh;

    ++itB;
  }
}



void GameObject::Display( Level& Level )
{

  if ( m_Rotateable )
  {
    theApp.RenderTextureSectionRotated2d( m_X + m_DisplayOffset.x - Level.m_OffsetX + 320, 
                                          m_Y + m_DisplayOffset.y - Level.m_OffsetY + 240, 
                                          theApp.m_TexSec[m_TexSection], 
                                          m_Angle,
                                          m_Color );
  }
  else
  {
    theApp.RenderTextureSection2d( m_X + m_DisplayOffset.x - Level.m_OffsetX + 320, 
                                   m_Y + m_DisplayOffset.y - Level.m_OffsetY + 240, 
                                   theApp.m_TexSec[m_TexSection],
                                   m_Color );
  }

}



void GameObject::Update( float ElapsedTime, Level& Level )
{
  std::list<Behaviour*>::iterator   itB( m_Behaviour.begin() );
  while ( itB != m_Behaviour.end() )
  {
    Behaviour*  pBeh( *itB );

    pBeh->Update( Level, ElapsedTime, this );

    ++itB;
  }
}



void GameObject::UpdateFixed( Level& Level )
{
  std::list<Behaviour*>::iterator   itB( m_Behaviour.begin() );
  while ( itB != m_Behaviour.end() )
  {
    Behaviour*  pBeh( *itB );

    pBeh->UpdateFixed( Level, this );

    ++itB;
  }
}



RECT GameObject::Bounds()
{

  RECT    bounds = m_Bounds;

  OffsetRect( &bounds, m_X, m_Y );
  return bounds;

}



void GameObject::OnEvent( Level& Level, const std::string& Event, GameObject* pObj, GameObject* pOtherObject )
{

  std::list<Behaviour*>::iterator   itB( m_Behaviour.begin() );
  while ( itB != m_Behaviour.end() )
  {
    Behaviour*  pBeh( *itB );

    pBeh->OnEvent( Level, Event, pObj, pOtherObject );

    ++itB;
  }

}