#pragma once

#include "GameObject.h"

#include <list>
#include <map>
#include <set>



class BehaviourPlayer;

class Level
{

  public:

    class Sector
    {
      public:


        std::list<GameObject*>      Objects;

        LD::TileType                Ground[5][5];


        Sector();
        void Clear();
    };


    std::map<std::pair<int,int>,Sector>       m_Sectors;

    int                       m_OffsetX;
    int                       m_OffsetY;

    float                     m_OffsetXFract;
    float                     m_OffsetYFract;

    std::multimap<int,GameObject*>    m_ObjectsToDisplay;

    std::map<int,std::list<std::string> > m_ExtraData;

    std::list<std::pair<std::string,int> >    m_DelayedEvents;

    GameObject*               m_pPlayer;
    BehaviourPlayer*          m_pPlayerBehaviour;

    std::string               m_DisplayText;
    POINT                     m_DisplayTextPos;
    float                     m_DisplayTextTime;

    std::string               m_Worldname;
    
    std::map<std::string,std::set<int> >    m_SingleKillItems;
    std::map<std::string,std::set<int> >    m_OpenItems;

    bool                      m_Done;
    int                       m_DoneTime;



    Level();
    ~Level();


    void                      Clear();

    void                      Display();
    void                      DisplaySector( int X, int Y, Sector& Sector );
    void                      DisplayVisibleBounds();

    void                      Update( float ElapsedTime );
    void                      UpdateFixed();

    bool                      Save( const std::string& Filename );
    bool                      Load( const std::string& Filename );

    void                      AddObject( GameObject* pObj );
    void                      RemoveObject( GameObject* pObj );
    void                      MoveObject( GameObject* pObj, int NewX, int NewY );

    bool                      AreaBlocked( GameObject* pMover, const RECT& rect, int DX, int DY );
    int                       WalkObject( GameObject* pObj, int DX, int DY );

    int                       NewID();

    GameObject*               FindObject( int X, int Y );


    int                       TileXFromScreenPos( int X );
    int                       TileYFromScreenPos( int Y );

    int                       TileXFromGlobalPos( int X );
    int                       TileYFromGlobalPos( int Y );

    void                      CheckVisibleCollisions();
    void                      CheckCollisions( GameObject* pObj, int X, int Y );
    void                      OnCollision( GameObject* pObj1, GameObject* pObj2 );

    bool                      CanCollide( GameObject* pObj1, GameObject* pObj2 );
    bool                      IsBlocking( LD::ObjectType Mover, LD::ObjectType Obstacle );

    GameObject*               FindObjectByID( int ID );

    void                      ParseExtraData( int ID );
    void                      HandleExtraData( const std::string& ExtraData, int ID );

};

